//#define SERIAL
//#if !defined(SERIAL)
  #include "MIDIUSB.h"
//#endif
#define CHANNEL 0x1
#define DEBOUNCE 2 // in millis
#define DEADBAND // 0~1023 
static const unsigned int maxRate = 0; // in millis
struct analog {
  String name;
  byte controllerNumber;
  unsigned int pin;
  unsigned int calibMin;
  unsigned int calibMax;
  unsigned int lastValue;
  unsigned int currentValue;
  unsigned int deadBand;
};
analog analogs[] = { {"pot", 20, A0, 290, 930, 0, 0, 0}, {"pedal1", 21, A1, 0, 530, 0, 0, 0}, {"pedal2", 22, A2, 0, 530, 0, 0, 0} };
unsigned int analogsCount = sizeof(analogs)/sizeof(analog);

struct digital {
  String name;
  byte midiNote;
  unsigned int pin;
  bool lastState;
  unsigned int debounce;
  long lastTriggered;
  
};

digital digitals[] = { {"A0", 33, 2, true, DEBOUNCE, 0}, {"F0", 29, 3, true, DEBOUNCE, 0}, {"D0", 26, 4, true, DEBOUNCE, 0}, {"G0", 31, 5, true, DEBOUNCE, 0},
                       {"E0", 28, 6, true, DEBOUNCE, 0}, {"B0", 35, 7, true, DEBOUNCE, 0}, {"C0", 24, 8, true, DEBOUNCE, 0}, {"Bb0", 34, 9, true, DEBOUNCE, 0},};
unsigned int digitalsCount = sizeof(digitals)/sizeof(digital);

void setup() {
  // setting up the digital inputs
  for (unsigned int i=0; i<digitalsCount; i++){
    pinMode(digitals[i].pin, INPUT_PULLUP);
    digitals[i].lastState = digitalRead(digitals[i].pin);
  }
  // setting up the analog inputs
  for (unsigned int i=0; i<analogsCount; i++) {
    analogs[i].currentValue = map(analogRead(analogs[i].pin), analogs[i].calibMin, analogs[i].calibMax, 0, 127);
  }
 // #if defined(SERIAL)
 //   Serial.begin(115200);
 //   delay(100);
 //   Serial.println("connected, "+String(analogsCount)+" analog inputs and "+String(digitalsCount)+" digital inputs configured");
 // #endif
}

void loop() {

//  Reading analog inputs
  for (unsigned int i=0; i<analogsCount; i++) {
    analogs[i].currentValue = constrain(analogRead(analogs[i].pin), analogs[i].calibMin, analogs[i].calibMax);
    analogs[i].currentValue = map(analogs[i].currentValue, analogs[i].calibMin,  analogs[i].calibMax, 0, 127);
    //if (analogs[i].lastValue != analogs[i].currentValue) {
      if (analogs[i].lastValue < analogs[i].currentValue - analogs[i].deadBand || analogs[i].lastValue > analogs[i].currentValue + analogs[i].deadBand) {
     //   #if defined(SERIAL)
     //     Serial.println(analogs[i].name + ": RAW"+String(analogRead(analogs[i].pin))+", SCALED : "+ String(analogs[i].currentValue));
        //analogs[i].lastValue = analogs[i].currentValue;
     //   #else
          byte ccValue = analogs[i].currentValue;
          controlChange(analogs[i].controllerNumber, ccValue);
     //   #endif
        delay(maxRate);
      }
    //}
    analogs[i].lastValue = analogs[i].currentValue;
  }

//  Reading digital inputs
  for (unsigned int i=0; i<digitalsCount; i++) {
    bool currentState = digitalRead(digitals[i].pin);
    // input has been pulled LOW
    if (currentState == LOW && digitals[i].lastState == HIGH && millis() - digitals[i].lastTriggered > digitals[i].debounce) {
      //#if defined(SERIAL)  
      //  Serial.println(digitals[i].name+":ON");
      //#else
        noteOn(digitals[i].midiNote, (byte) 127);
      //#endif
      digitals[i].lastTriggered = millis();
      
    }
    // input has been pulled HIGH
    if (currentState == HIGH && digitals[i].lastState == LOW && millis() - digitals[i].lastTriggered > digitals[i].debounce) {
      //#if defined(SERIAL)
        Serial.println(digitals[i].name+":OFF");
      //#else
        noteOff(digitals[i].midiNote);
      //#endif
      digitals[i].lastTriggered = millis();
    }
    digitals[i].lastState = currentState;
  }
  //#if !defined(SERIAL)
    MidiUSB.flush();
  //#endif
  delay(5);
}

//#if !defined(SERIAL)
  void noteOn(byte pitch, byte velocity) {
    midiEventPacket_t noteOn = {0x09, 0x90 | CHANNEL, pitch, velocity};
    MidiUSB.sendMIDI(noteOn);
  }
  
  void noteOff(byte pitch) {
    midiEventPacket_t noteOff = {0x08, 0x80 | CHANNEL, pitch, 0x0};
    MidiUSB.sendMIDI(noteOff);
  }
  
  void controlChange(byte control, byte value) {
    midiEventPacket_t event = {0x0B, 0xB0 | CHANNEL, control, value};
    MidiUSB.sendMIDI(event);
  }
//#endif
